def extend_sloth(cls, extension):
    '''Add SQLite logging for build events.

    :param cls: base :mod:`Sloth <sloth_ci.sloth.Sloth>` class to extend
    :param extension: ``{'build_history': {'module': 'db.builds', 'db': self.db_path, 'table': 'build_history'}}``

    :returns: extended :mod:`Sloth <sloth_ci.sloth.Sloth>` class
    '''

    from .util import SqliteHandler


    class Sloth(cls):
        def __init__(self, config):
            super().__init__(config)

            db_build_log_config = extension['config']

            db_build_log_handler = SqliteHandler(db_build_log_config['db'], db_build_log_config['table'])

            self.build_logger.addHandler(db_build_log_handler)

            self.log_handlers[extension['name']] = db_build_log_handler

        def stop(self):
            super().stop()
            self.build_logger.removeHandler(self.log_handlers.pop(extension['name']))


    return Sloth



'''Describe your extension.


Installation
------------

.. code-block:: bash

    $ pip install sloth-ci.ext.stdout


Usage
-----

.. code-block:: yaml
    :caption: stdout.yml

    extensions:
        stdout:
            # Use the module sloth_ci.ext.stdout.
            module: stdout
'''

__title__ = 'sloth-ci.ext.stdout'
__description__ = 'Get stdout and stderr of Sloth CI apps.'
__version__ = '0.0.1'
__author__ = 'Konstantin Molchanov'
__author_email__ = 'moigagoo@live.com'
__license__ = 'MIT'


def extend_bed(cls, extension):
    '''Add the ``stdout`` endpoint.

    :param cls: the base ``sloth_ci.bed.Bed`` class
    :param extension: ``{'name': '{extension}', 'config': {param1: value2, param2: value2, ...}}``, extracted from the server config
    '''

    import cherrypy

    import sqlite3

    class Bed(cls):

        def __init__(self, config):
            super().__init__(config)

            self.actions['stdout'] = self.stdout

        def stdout(self, kwargs):
            '''Get paginated app stdout and stderr output.'''

            if not self.db_path:
                raise cherrypy.HTTPError(
                    501, "This Sloth server doesn't have a database to store logs")

            listen_point = kwargs.get('listen_point')

            if not listen_point:
                raise cherrypy.HTTPError(
                    400, 'Missing parameter "listen_point"')

            try:
                if listen_point not in self.sloths:
                    raise KeyError(listen_point)

                from_page = int(kwargs.get('from_page', 1))
                to_page = int(kwargs.get('to_page', from_page))
                per_page = int(kwargs.get('per_page', 10))

                connection = sqlite3.connect(self.db_path)
                cursor = connection.cursor()

                query = 'SELECT * FROM app_logs \
                    WHERE logger_name=? \
                    AND level_number = 10 \
                    ORDER BY timestamp DESC \
                    LIMIT ? OFFSET ?'

                query_params = (
                    listen_point + '.exec',
                    (to_page - from_page + 1) * per_page,
                    (from_page - 1) * per_page
                )

                cursor.execute(query, query_params)

                column_names = [column[0] for column in cursor.description]

                stdout = [dict(zip(column_names, record))
                          for record in cursor.fetchall()]

                connection.close()

                cherrypy.response.status = 200

                return stdout

            except KeyError as e:
                raise cherrypy.HTTPError(404, 'Listen point %s not found' % e)

            except Exception as e:
                raise cherrypy.HTTPError(
                    500, 'Failed to get app stdout: %s' % e)

    return Bed


def extend_cli(cls, extension):
    '''Modify ``sloth_ci.cli.CLI`` to add new ``sci`` commands.

    :param cls: the base ``sloth_ci.cli.CLI`` class
    :param extension: ``{'name': 'stdout', 'config': {param1: value2, param2: value2, ...}}``, extracted from the server config
    '''

    from time import ctime

    from tabulate import tabulate
    from cliar import add_aliases

    class CLI(cls):

        @add_aliases(['out', 'stderr'])
        def stdout(self, app, from_page: int=1, to_page: int=1, per_page: int=10, raw=False):
            '''get stdout and strerr for APP'''

            response = self.send_api_request(
                {
                    'action': 'stdout',
                    'listen_point': app,
                    'from_page': from_page,
                    'to_page': to_page,
                    'per_page': per_page,
                }
            )

            if response.status_code == 200:
                rows = [
                    [
                        ctime(record['timestamp']),
                        record['message'],
                    ] for record in response.content
                ]

                headers = [
                    'Timestamp',
                    'Message'
                ]

                if raw:
                    pass
                else:
                    table = tabulate(
                        self.colorize(rows, based_on_column=-1),
                        headers=headers
                    )

                print(table)

            else:
                print('Failed to get app stdout: %s' % response.content)

    return CLI



def extend_bed(cls, extension):
    '''Add routes and handlers for the API methods.

    :param cls: base :class:`Bed <sloth_ci.bed.Bed>` class to be extended
    :param extension: ``{'api': {'module': 'api'}}``

    :returns: extended :class:`Bed <sloth_ci.bed.Bed>` class
    '''


    import cherrypy

    from cherrypy.lib.auth_basic import checkpassword_dict

    from yaml import load

    import sqlite3

    from ... import __version__


    class Bed(cls):
        def __init__(self, config):
            super().__init__(config)

            self.actions = {
                'create': self.create,
                'bind': self.bind,
                'remove': self.remove,
                'trigger': self.trigger,
                'info': self.info,
                'list': self.list,
                'logs': self.logs,
                'history': self.history,
                'version': self.version,
                'restart': self.restart,
                'stop': self.stop
            }

        def _setup_routing(self):
            '''Setup routing for the API endpoint.'''

            super()._setup_routing()

            for alias in ('auth', 'api_auth'):
                auth = self.config.get(alias)
                if auth:
                    break
            else:
                raise KeyError('API credentials are missing in the config file.')

            listener = self._make_listener({auth['login']: auth['password']})

            self._dispatcher.connect('api', '/', listener)

        def _handle_error(self, status, message, traceback, version):
            return message

        def _make_listener(self, auth_dict, realm='sloth-ci'):
            '''Get a basic-auth-protected listener function for the API endpoint.

            :param auth_dict: {user: password} dict for authentication
            :param realm: mandatory param for basic auth

            :returns: a CherryPy listener function
            '''

            @cherrypy.expose
            @cherrypy.tools.auth_basic(checkpassword=checkpassword_dict(auth_dict), realm=realm)
            @cherrypy.tools.json_out()
            def listener(action, **kwargs):
                '''Listen to and route API requests.

                An API request is a GET or POST HTTP request with one mandatory param ``action`` and
                additional params for the given action.

                Params can be provided in the URL or as form data.

                :param action: string corresponding to one of the available API methods.
                :param kwargs: params for the given action.
                '''

                cherrypy.request.error_page = {'default': self._handle_error}

                try:
                    return self.actions[action](kwargs)

                except KeyError as e:
                    raise cherrypy.HTTPError(404, 'Action %s not found' % e)

            return listener

        def bind(self, kwargs):
            '''Bind an app with a config file.'''

            listen_point = kwargs.get('listen_point')
            config_file = kwargs.get('config_file')

            if not listen_point:
                raise cherrypy.HTTPError(400, 'Missing parameter "listen_point"')

            if not config_file:
                raise cherrypy.HTTPError(400, 'Missing parameter config_file')

            try:
                self.bind_to_file(listen_point, config_file)

                cherrypy.response.status = 200

                return None

            except KeyError as e:
                raise cherrypy.HTTPError(404, 'Listen point %s not found' % e)

            except FileNotFoundError as e:
                raise cherrypy.HTTPError(404, 'File %s not found' % e)

            except ValueError:
                raise cherrypy.HTTPError(500, 'Config mismatch')

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to bind config file to app: %s' % e)

        def create(self, kwargs):
            '''Create an app from the given config string.'''

            config_string = kwargs.get('config_string')

            if not config_string:
                raise cherrypy.HTTPError(400, 'Missing parameter config_string')

            try:
                listen_point = self.create_from_config(load(config_string))

                cherrypy.response.status = 201

                return listen_point

            except TypeError:
                raise cherrypy.HTTPError(500, '"%s" is not a valid config string' % config_string)

            except KeyError as e:
                raise cherrypy.HTTPError(500, 'The "%s" param is missing in the config' % e)

            except ValueError as e:
                raise cherrypy.HTTPError(409, 'Listen point %s is taken' % e)

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to create app: %s' % e)

        def history(self, kwargs):
            '''Get paginated app build history from the database.'''

            if not self.db_path:
                raise cherrypy.HTTPError(501, "This Sloth server doesn't have a database to store build history")

            listen_point = kwargs.get('listen_point')

            if not listen_point:
                raise cherrypy.HTTPError(400, 'Missing parameter "listen_point"')

            try:
                if not listen_point in self.sloths:
                    raise KeyError(listen_point)

                from_page = int(kwargs.get('from_page', 1))
                to_page = int(kwargs.get('to_page', from_page))
                per_page = int(kwargs.get('per_page', 10))
                level = int(kwargs.get('level', 20))

                connection = sqlite3.connect(self.db_path)
                cursor = connection.cursor()

                query = 'SELECT * FROM build_history \
                    WHERE logger_name=? \
                    AND level_number >= ? \
                    ORDER BY timestamp DESC \
                    LIMIT ? OFFSET ?'

                query_params = (
                    listen_point + '.build',
                    level,
                    (to_page - from_page + 1) * per_page,
                    (from_page - 1) * per_page
                )

                cursor.execute(query, query_params)

                column_names = [column[0] for column in cursor.description]

                history = [dict(zip(column_names, record)) for record in cursor.fetchall()]

                connection.close()

                cherrypy.response.status = 200

                return history

            except KeyError as e:
                raise cherrypy.HTTPError(404, 'Listen point %s not found' % e)

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to get app build history: %s' % e)

        def info(self, kwargs):
            '''Get information about an app on an given listen point.'''

            listen_point = kwargs.get('listen_point')

            if not listen_point:
                raise cherrypy.HTTPError(400, 'Missing parameter "listen_point"')

            try:
                if not listen_point in self.sloths:
                    raise KeyError(listen_point)

                info = {
                    'config_file': self.config_files.get(listen_point)
                }

                if self.db_path:
                    last_build_info = self.history({
                        'listen_point': listen_point,
                        'per_page': 1
                    })

                    if last_build_info:
                        last_build_status_message = last_build_info[0]['message']
                        last_build_status_level = last_build_info[0]['level_name']
                        last_build_timestamp = last_build_info[0]['timestamp']

                    else:
                        last_build_status_message = 'Never triggered'
                        last_build_status_level = 'Never triggered'
                        last_build_timestamp = 0

                    info['last_build_status_message'] = last_build_status_message
                    info['last_build_status_level'] = last_build_status_level
                    info['last_build_timestamp'] = last_build_timestamp

                else:
                    info['last_build_status_message'] = 'Not available'
                    info['last_build_status_level'] = 'Not available'
                    info['last_build_timestamp'] = 0

                cherrypy.response.status = 200

                return info

            except KeyError as e:
                raise cherrypy.HTTPError(404, 'Listen point %s not found' % e)

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to get app info: %s' % e)

        def list(self, kwargs):
            '''List all available apps.'''

            try:
                app_list = self.sloths.keys()

                cherrypy.response.status = 200

                return sorted(app_list)

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to get app list: %s' % e)

        def logs(self, kwargs):
            '''Get paginated app logs from the database.'''

            if not self.db_path:
                raise cherrypy.HTTPError(501, "This Sloth server doesn't have a database to store logs")

            listen_point = kwargs.get('listen_point')

            if not listen_point:
                raise cherrypy.HTTPError(400, 'Missing parameter "listen_point"')

            try:
                if listen_point not in self.sloths:
                    raise KeyError(listen_point)

                from_page = int(kwargs.get('from_page', 1))
                to_page = int(kwargs.get('to_page', from_page))
                per_page = int(kwargs.get('per_page', 10))
                level = int(kwargs.get('level', 20))

                connection = sqlite3.connect(self.db_path)
                cursor = connection.cursor()

                query = 'SELECT * FROM app_logs \
                    WHERE (logger_name=? OR logger_name=? OR logger_name=?) \
                    AND level_number >= ? \
                    ORDER BY timestamp DESC \
                    LIMIT ? OFFSET ?'

                query_params = (
                    listen_point,
                    listen_point + '.exec',
                    listen_point + '.build',
                    level,
                    (to_page - from_page + 1) * per_page,
                    (from_page - 1) * per_page
                )

                cursor.execute(query, query_params)

                column_names = [column[0] for column in cursor.description]

                logs = [dict(zip(column_names, record)) for record in cursor.fetchall()]

                connection.close()

                cherrypy.response.status = 200

                return logs

            except KeyError as e:
                raise cherrypy.HTTPError(404, 'Listen point %s not found' % e)

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to get app logs: %s' % e)

        def remove(self, kwargs):
            '''Remove an app on the given listen point.'''

            listen_point = kwargs.get('listen_point')

            if not listen_point:
                raise cherrypy.HTTPError(400, 'Missing parameter "listen_point"')

            try:
                super().remove(listen_point)

                cherrypy.response.status = 204

                return None

            except KeyError as e:
                raise cherrypy.HTTPError(404, 'Listen point %s not found' % e)

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to remove app: %s' % e)

        def restart(self, kwargs):
            '''Ask the Sloth CI server to restart.'''

            try:
                self.bus.restart()

                cherrypy.response.status = 202

                return None

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to restart Sloth CI server: %s' % e)

        def stop(self, kwargs):
            '''Ask the Sloth CI server to stop.'''

            try:
                self.bus.exit()

                cherrypy.response.status = 202

                return None

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to stop Sloth CI server: %s' % e)

        def trigger(self, kwargs):
            '''Trigger action of an app on the given listen point.'''

            listen_point = kwargs.get('listen_point')

            if not listen_point:
                raise cherrypy.HTTPError(400, 'Missing parameter "listen_point"')

            try:
                params = {key: kwargs[key] for key in kwargs if key not in ('listen_point', 'wait')}

                sloth = self.sloths[listen_point]

                sloth.process(params)

                if not kwargs.get('wait'):
                    cherrypy.response.status = 202
                    return None

                else:
                    sloth.queue_processor.join()
                    return self.info({'listen_point': listen_point})['last_build_status_message']

            except KeyError as e:
                raise cherrypy.HTTPError(404, 'Listen point %s not found' % e)

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to trigger app actions: %s' % e)

        def version(self, kwargs):
            '''Get the Sloth CI app version.'''

            try:
                version = __version__

                cherrypy.response.status = 200

                return version

            except Exception as e:
                raise cherrypy.HTTPError(500, 'Failed to get Sloth CI server version: %s' % e)


    return Bed


'''Lightweight tool that runs commands when you push to GitHub, Bitbucket, or Gitlab.

Just type the commands you'd run manually and let Sloth CI run them for you!
'''

__title__ = 'sloth-ci'
__description__ = 'Lightweight CI tool'
__version__ = '2.0.6'
__author__ = 'Konstantin Molchanov'
__author_email__ = 'moigagoo@live.com'
__license__ = 'MIT'



from importlib import import_module

from argparse import ArgumentParser

from yaml import load

from cliar import CLI as CliarCLI
from cliar import ignore as ignore_method

from .bed import Bed

from . import __version__


class CLI(CliarCLI):
    '''Welcome to Sloth CI CLI!

Run "sci start" to start the server.'''

    def __new__(cls):
        '''Apply extenions before creating a CLI instance.

        The built-in API extension is always applied before any custom ones.
        '''

        api_extension = {
            'api': {
                'module': 'api'
            }
        }

        PreExtendedCLI, pre_errors = cls.extend(api_extension)

        parser = ArgumentParser(add_help=False)
        parser.add_argument('-config', default='./sloth.yml')
        config = parser.parse_known_args()[0].config

        try:
            cls.config = load(open(config))

        except FileNotFoundError:
            print('Please run inside a dir with "sloth.yml" or specify the server config with "-c."')
            exit()

        except Exception as e:
            print('Failed to parse the config file: %s' % e)
            exit()

        ExtendedCLI, errors = PreExtendedCLI.extend(cls.config.get('extensions'))

        for error in pre_errors + errors:
            print(error)

        return super().__new__(ExtendedCLI)

    @classmethod
    @ignore_method
    def extend(cls, extensions):
        '''Sequentially chain-inherit CLI classes from extensions.

        The first extension's CLI class inherits from the base CLI class and becomes the base class, then the second one inherits from it, and so on.

        :param extensions: list of extensions to load.

        :returns: `ExtendedCLI` is a CLI class inherited from all extensions' CLI classes; `errors` is the list of errors raised during extension loading.
        '''

        ExtendedCLI = cls
        errors = []

        if extensions:
            for extension_name, extension_config in extensions.items():
                try:
                    ext = import_module('.ext.%s' % extension_config['module'], package=__package__)

                    ExtendedCLI = ext.extend_cli(ExtendedCLI, {
                            'name': extension_name,
                            'config': extension_config
                        }
                    )

                except AttributeError as e:
                    pass

                except Exception as e:
                    errors.append('Could not load extension %s: %s' % (extension_name, e))

        return ExtendedCLI, errors

    def _root(self, config='./sloth.yml', version=False):
        if version:
            print(__version__)
            exit()

    def start(self):
        '''start server'''

        try:
            print(
                'Starting Sloth CI on http://%s:%d'
                % (self.config['host'], self.config['port'])
            )

            Bed(self.config).start()

        except Exception as e:
            print('Failed to start Sloth CI: %s' % e)

def main():
    CLI().parse()



from logging import Handler

import sqlite3


class SqliteHandler(Handler):
    '''SQLite handler for the Python logging module.'''

    def __init__(self, db, table):
        super().__init__()

        self.connection = sqlite3.connect(db, check_same_thread=False)
        self.cursor = self.connection.cursor()

        self.table = table

        query = 'CREATE TABLE IF NOT EXISTS %s (timestamp, logger_name, level_name, level_number, message)' % self.table

        self.cursor.execute(query)
        self.connection.commit()

    def emit(self, record):
        try:
            query = 'INSERT INTO %s VALUES (?, ?, ?, ?, ?)' % self.table
            query_params = (record.created, record.name, record.levelname, record.levelno, record.msg)

            self.cursor.execute(query, query_params)
            self.connection.commit()

        except Exception:
            self.handleError(record)

    def close(self):
        self.connection.close()


from subprocess import Popen, PIPE, TimeoutExpired
from threading import Thread
from collections import deque
from importlib import import_module

from cherrypy import HTTPError

import logging


class Sloth:
    '''Base Sloth class.

    Each instance represents a separate Sloth app,
    with its own config, log, action queue, and queue processor.

    Each app listens for incoming requests on its own URL path.
    '''

    def __init__(self, config):
        self.config = config

        for alias in ('id', 'name', 'listen_point'):
            listen_point = self.config.get(alias)
            if listen_point:
                break
        else:
            raise KeyError('id')

        self.listen_point = listen_point

        self.logger = logging.getLogger(self.listen_point)
        self.logger.setLevel(logging.DEBUG)

        self.build_logger = self.logger.getChild('build')
        self.exec_logger = self.logger.getChild('exec')

        self.log_handlers = {}

        self.queue = deque()
        self._queue_lock = False

        self.queue_processor = None
        self._processing_lock = False

    @classmethod
    def extend(cls, extensions):
        '''Sequentially chain-inherit Sloth classes from extensions.

        The first extension's Sloth class inherits from the base Sloth class and becomes the base class, then the second one inherits from it, and so on.

        :param extensions: dict of extensions to load.

        :returns: `ExtendedSloth` is a Sloth class inherited from all extensions' Sloth classes; `errors` is the list of errors raised during extension loading.
        '''

        ExtendedSloth = cls
        errors = []

        if extensions:
            for extension_name, extension_config in extensions.items():
                try:
                    ext = import_module('.ext.%s' % extension_config['module'], package=__package__)

                    ExtendedSloth = ext.extend_sloth(ExtendedSloth, {
                            'name': extension_name,
                            'config': extension_config
                        }
                    )

                except AttributeError as e:
                    pass

                except Exception as e:
                    errors.append('Could not load extension %s: %s' % (extension_name, e))

        return ExtendedSloth, errors

    def handle(self, request):
        '''Validate, extract, and process incoming payload.

        :param request: a cherrypy.request instance
        '''

        self.logger.debug('Payload received from %s - %s' % (
                request.remote.ip,
                request.headers.get('User-Agent', '')
            )
        )

        try:
            provider_section = self.config.get('provider')

            if provider_section:
                provider, provider_data = provider_section.copy().popitem()

            else:
                raise HTTPError(400, 'No provider set, declining all payloads')

            validator = import_module('.validators.%s' % provider, package=__package__)

        except ImportError as e:
            self.logger.critical('No matching validator found: %s' % e)
            raise HTTPError(500, 'No matching validator found: %s' % e)

        validation_data = provider_data or {}

        status, message, param_dicts = validator.validate(request, validation_data)

        self.logger.debug(message)

        if status == 200:
            self.logger.info('Valid payload received')

        else:
            raise HTTPError(status, message)

        for params in param_dicts:
            self.process(params)

    def process(self, validator_params):
        '''Queue execution of actions with certain params.

        Params are taken from the ``params`` config section and extracted from the incoming payload.

        :param validator_params: params extracted from the payload
        '''

        params = dict(self.config.get('params', {}), **validator_params)

        if not self._queue_lock:
            self.queue.append(params)

        if not self.queue_processor or not self.queue_processor.is_alive():
            self.queue_processor = Thread(target=self.process_queue, name=self.listen_point)
            self.queue_processor.start()

    def process_queue(self):
        '''Processes execution queue in a separate thread.

        :returns: True if successful, exception otherwise
        '''

        actions = self.config.get('actions')

        if actions:
            while self.queue:
                if self._processing_lock:
                    self.exec_logger.warning('Queue processing interrupted')
                    break

                params = self.queue.popleft()

                try:
                    self.run_build(actions, params)

                except:
                    pass

        return True

    def run_build(self, actions, params):
        '''Run a build with the given params.

        :param actions: actions in this build
        :param params: params used by the actions

        :returns: True if successful, exception otherwise
        '''

        errors = []

        self.build_logger.debug('Triggered, actions in queue: %d' % len(actions))

        self.exec_logger.debug('Params: %s' % params)

        for action in actions:
            try:
                action_with_params = action.format_map(params)

                self.exec_logger.info('Executing action: %s' % action_with_params)

                self.execute(action_with_params)

            except KeyError as e:
                self.exec_logger.error('Missing param: %s' % e)
                errors.append(e)

            except Exception as e:
                self.exec_logger.error('Execution failed: %s' % e)
                errors.append(e)

            finally:
                if errors and self.config.get('stop_on_first_fail'):
                    self.build_logger.error('Failed on action "%s": %s' % (action, errors[0]))
                    raise errors[0]

        if not errors:
            self.build_logger.info('Completed %d/%d' % (len(actions), len(actions)))

        elif len(errors) == len(actions):
            self.build_logger.warning('None completed: %d/%d' % (len(actions) - len(errors), len(actions)))

        else:
            self.build_logger.warning('Partially completed: %d/%d' % (len(actions) - len(errors), len(actions)))

    def execute(self, action):
        '''Executes an action in an ordinary Popen.

        :param action: action to be executed

        :returns: True if successful, exception otherwise
        '''

        try:
            process = Popen(
                action,
                shell=True,
                cwd=self.config.get('work_dir') or '.',
                stdout=PIPE,
                stderr=PIPE
            )

            stdout, stderr = process.communicate(timeout=self.config.get('exec_timeout'))

            self.exec_logger.debug('stdout: %s' % bytes.decode(stdout))
            self.exec_logger.debug('stderr: %s' % bytes.decode(stderr))

            if process.returncode:
                raise RuntimeError('Exit code: %d' % process.returncode)

            self.exec_logger.info('Finished')

            return True

        except TimeoutExpired:
            process.kill()

            stdout, stderr = process.communicate()

            self.exec_logger.debug(bytes.decode(stdout))
            self.exec_logger.debug(bytes.decode(stderr))

            raise

        except Exception:
            raise

    def stop(self):
        '''Gracefully stop the queue processor.

        New payloads are not added to the queue, existing actions will be finished.
        '''

        self._queue_lock = True
        self.logger.info('Stopped')

    def kill(self):
        '''Immediately stop processing the queue.'''

        self.stop()

        self._processing_lock = True
        self.logger.warning('Killed')



from setuptools import setup, find_packages

import sloth_ci


try:
    long_description = open('README.rst').read()

except:
    long_description = sloth_ci.__doc__


setup(
    name=sloth_ci.__title__,
    version=sloth_ci.__version__,
    author=sloth_ci.__author__,
    description=sloth_ci.__description__,
    long_description=long_description,
    author_email=sloth_ci.__author_email__,
    url='https://sloth-ci.com',
    packages=find_packages(exclude=['docs']),
    install_requires=[
        'CherryPy',
        'Routes',
        'PyYAML',
        'requests',
        'cliar',
        'tabulate'
    ],
    extras_require={
        'Colored logs': ['colorama'],
        'Logging to files':  ['sloth-ci.ext.file_logs'],
        'Developer tools': ['sloth-ci.ext.devtools']
    },
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3 :: Only',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Unix',
        'Framework :: CherryPy'],
    entry_points={
        'console_scripts': [
            'sloth-ci=sloth_ci.cli:main',
            'sci=sloth_ci.cli:main'
        ]
    }
)
